@@ -50,7 +50,7 @@ use Perlbal::ClientProxy;
 
 # if this is made too big, (say, 128k), then perl does malloc instead
 # of using its slab cache.
-use constant BACKEND_READ_SIZE => 61449;  # 60k, to fit in a 64k slab
+use constant BACKEND_READ_SIZE => 61440;  # 60k, to fit in a 64k slab
 
 # keys set here when an endpoint is found to not support persistent
 # connections and/or the OPTIONS method
@@ -474,19 +474,19 @@ sub handle_response { # : void
     # special cases:  reproxying and retrying after server errors:
     if ((my $rep = $hd->header('X-REPROXY-FILE')) && $self->may_reproxy) {
         # make the client begin the async IO while we move on
-        $client->start_reproxy_file($rep, $hd);
         $self->next_request;
+        $client->start_reproxy_file($rep, $hd);
         return;
     } elsif ((my $urls = $hd->header('X-REPROXY-URL')) && $self->may_reproxy) {
+        $self->next_request;
         $self->{service}->add_to_reproxy_url_cache($rqhd, $hd)
             if $reproxy_cache_for;
-        $client->start_reproxy_uri($self->{res_headers}, $urls);
-        $self->next_request;
+        $client->start_reproxy_uri($hd, $urls);
         return;
     } elsif ((my $svcname = $hd->header('X-REPROXY-SERVICE')) && $self->may_reproxy) {
-        $self->{client} = undef;
-        $client->start_reproxy_service($self->{res_headers}, $svcname);
         $self->next_request;
+        $self->{client} = undef;
+        $client->start_reproxy_service($hd, $svcname);
         return;
     } elsif ($res_code == 500 &&
              $rqhd->request_method =~ /^GET|HEAD$/ &&
@@ -504,10 +504,6 @@ sub handle_response { # : void
     my $res_source = $client->{primary_res_hdrs} || $hd;
     my $thd = $client->{res_headers} = $res_source->clone;
 
-    # setup_keepalive will set Connection: and Keep-Alive: headers for us
-    # as well as setup our HTTP version appropriately
-    $client->setup_keepalive($thd);
-
     # if we had an alternate primary response header, make sure
     # we send the real content-length (from the reproxied URL)
     # and not the one the first server gave us
@@ -520,10 +516,18 @@ sub handle_response { # : void
 
         # also update the response code, in case of 206 partial content
         my $rescode = $hd->response_code;
-        $thd->code($rescode) if $rescode == 206 || $rescode == 416;
+        if ($rescode == 206 || $rescode == 416) {
+            $thd->code($rescode);
+            $thd->header('Accept-Ranges', $hd->header('Accept-Ranges')) if $hd->header('Accept-Ranges');
+            $thd->header('Content-Range', $hd->header('Content-Range')) if $hd->header('Content-Range');
+        }
         $thd->code(200) if $thd->response_code == 204;  # upgrade HTTP No Content (204) to 200 OK.
     }
 
+    # setup_keepalive will set Connection: and Keep-Alive: headers for us
+    # as well as setup our HTTP version appropriately
+    $client->setup_keepalive($thd);
+
     print "  writing response headers to client\n" if Perlbal::DEBUG >= 3;
     $client->write($thd->to_string_ref);
 
